------ Differences with Spagic 2.3.0 ------
CREATE SEQUENCE seq_search_query;

Create table search_query (
   id_query Integer NOT NULL ,
   label Varchar(100) NOT NULL UNIQUE,
   query Text NOT NULL,
Primary Key (id_query));

-- Create view section
CREATE VIEW bc_input_messages AS
  SELECT s.id_step AS id_binding_component, COUNT(logJournal.id_log) AS input_messages 
	FROM step s, process p, transition_instance t, log_journal logJournal,property_step propertyStep, property pty  	
	WHERE s.id_process = p.id_process AND s.id_process = (SELECT max(pSub.id_process) AS idProcess FROM process pSub WHERE pSub.name=p.name)				
				AND s.id_step = t.id_step_source
				AND t.id_journal = logJournal.id_journal 				
				AND s.id_step=propertyStep.id_step 				
				AND propertyStep.id_property=pty.id_property
				AND pty.code='im.servicebindingname'
				AND ((s.is_start AND logJournal.id_state IN (0, 3)) 
						OR (pty.value IN ('JBI-WebServicePipeline','JBI-TCPIP-Pipeline','JBI-CXF-Pipeline') AND logJournal.id_state=3 )
						) 				
	group by s.id_step;

CREATE VIEW bc_output_messages AS
  SELECT s.id_step AS id_binding_component, COUNT(logJournal.id_log) AS output_messages 
	FROM step s, process p, transition_instance t, log_journal logJournal,property_step propertyStep, property pty  	
	WHERE s.id_process = p.id_process AND s.id_process = (SELECT max(pSub.id_process) AS idProcess FROM process pSub WHERE pSub.name=p.name)								
				AND t.id_journal = logJournal.id_journal 				
				AND s.id_step=propertyStep.id_step 				
				AND propertyStep.id_property=pty.id_property
				AND pty.code='im.servicebindingname'
				AND ((s.is_end AND logJournal.id_state=1 AND s.id_step = t.id_step_target)
						OR (pty.value IN ('JBI-WebServicePipeline','JBI-TCPIP-Pipeline','JBI-CXF-Pipeline') AND s.id_step = t.id_step_source AND logJournal.id_state=4 )
            OR (s.is_start AND logJournal.id_state = 4 AND s.id_step = t.id_step_source)						
						) 				
	group by s.id_step;
	
CREATE VIEW bc_error_messages AS
  SELECT s.id_step AS id_binding_component, COUNT(logJournal.id_log) AS error_messages 
	FROM step s, process p, transition_instance t, log_journal logJournal,property_step propertyStep, property pty  	
	WHERE s.id_process = p.id_process AND s.id_process = (SELECT max(pSub.id_process) AS idProcess FROM process pSub WHERE pSub.name=p.name)								
				AND t.id_journal = logJournal.id_journal 				
				AND s.id_step=propertyStep.id_step 				
				AND propertyStep.id_property=pty.id_property
				AND pty.code='im.servicebindingname'
				AND ((s.is_start AND s.id_step = t.id_step_source AND logJournal.id_state IN (2, 5))
						OR(s.is_end  AND s.id_step = t.id_step_target AND logJournal.id_state IN (2, 5))
						OR (pty.value IN ('JBI-WebServicePipeline','JBI-TCPIP-Pipeline','JBI-CXF-Pipeline') AND logJournal.id_state=5
								AND (s.id_step = t.id_step_target OR s.id_step = t.id_step_source))
						) 				
	group by s.id_step;

-- Differences with Spagic 2.3.0 --

ALTER TABLE process_instance DROP INDEX idx_correlation_id,
 ADD INDEX idx_process_instance_01 (id_correlation);
/* Cannot be unique due to restarted instances */
Create Index idx_transition_instance_01 ON transition_instance using btree (id_message);
Create Index idx_log_journal_01 ON log_journal using btree (id_journal);
Create Index idx_log_journal_02 ON log_journal using btree (id_state);


-- Differences with Spagic 2.4.0 --

ALTER TABLE process_instance ADD COLUMN id_parent_process_instance Bigint DEFAULT NULL AFTER id_process_instance;
ALTER TABLE process_instance add foreign key (id_parent_process_instance) references process_instance (id_process_instance) on update restrict on delete restrict;

Create table sub_process 
(
	id_process Bigint NOT NULL,
	id_subprocess Bigint NOT NULL,
	descr Varchar(50),
 primary key (id_process,id_subprocess)
) Without Oids;

ALTER TABLE sub_process add foreign key (id_process) references process (id_process) on delete  restrict on update  restrict;
ALTER TABLE sub_process add foreign key (id_subprocess) references process (id_process) on delete  restrict on update  restrict;

INSERT INTO type_service (id_ts, value) VALUES(6, 'JBPM-PROCESS');

INSERT INTO transition_state (id_state,state_value) VALUES(21, 'JBPM_START_ACTIVITY');
INSERT INTO transition_state (id_state,state_value) VALUES(22, 'JBPM_END_ACTIVITY');
INSERT INTO transition_state (id_state,state_value) VALUES(23, 'JBPM_FAULTED_ACTIVITY');


ALTER TABLE process_instance ADD COLUMN id_process_execution_state Bigint NOT NULL Default 0 AFTER id_pool;
ALTER TABLE process ADD COLUMN id_monitoring_type Bigint NOT NULL Default 0 AFTER monitor_enabled;


Create table process_execution_state
(
	id_process_execution_state Double precision NOT NULL,
	state_value Varchar(100),
 primary key (id_process_execution_state)
) Without Oids;

Create table monitoring_type
(
	id_monitoring_type Double precision NOT NULL,
	type_value Varchar(100),
 primary key (id_monitoring_type)
) Without Oids;

INSERT INTO pool_type (id_pool_type,id_rule_det_pool_type,name,description,pool_type_version) VALUES(nextval('seq_pool_type'),NULL,'ANONYMOUS','POOL TYPE FOR DYNAMIC LINKS','0');

INSERT INTO process_execution_state (id_process_execution_state,state_value) VALUES(0, 'NORMAL_EXECUTION');
INSERT INTO process_execution_state (id_process_execution_state,state_value) VALUES(1, 'PAUSE_EXECUTION');
INSERT INTO process_execution_state (id_process_execution_state,state_value) VALUES(2, 'STOP_EXECUTION');

INSERT INTO monitoring_type (id_monitoring_type,type_value) VALUES(0, 'ASYNCHRONOUS');
INSERT INTO monitoring_type (id_monitoring_type,type_value) VALUES(1, 'SYNCHRONOUS');


ALTER TABLE process_instance add  foreign key (id_process_execution_state) references process_execution_state (id_process_execution_state) on update restrict on delete restrict;

ALTER TABLE process add  foreign key (id_monitoring_type) references monitoring_type (id_monitoring_type) on update restrict on delete restrict;

ALTER TABLE step ADD COLUMN has_conditional_exit Boolean;


-- Transition_data as CLOB in Transition Instance
-- Old data in the columns transition_data and transition_data_out
-- will be lost
BEGIN;
ALTER TABLE transition_instance DROP COLUMN transition_data;
ALTER TABLE transition_instance ADD COLUMN transition_data Text;
COMMIT; 
BEGIN;
ALTER TABLE transition_instance DROP COLUMN transition_data_out;
ALTER TABLE transition_instance ADD COLUMN transition_data_out Text;
COMMIT; 

BEGIN;
ALTER TABLE transition_instance DROP COLUMN ID_EXT_PROCESS_INSTANCE;
ALTER TABLE transition_instance ADD COLUMN ID_EXT_PROCESS_INSTANCE Bigint;
COMMIT;


-- 2.5.0 performance improvement in backup
Create table temporary_process_instance
(
	id_process_instance Bigint NOT NULL,
 primary key (id_process_instance)
) Without Oids;

ALTER TABLE transition_instance ADD COLUMN runtime_warning Boolean DEFAULT FALSE AFTER attachment_out;

-- 2.5.1 add documentation to a workflow step

ALTER TABLE step ADD COLUMN documentation BLOB  DEFAULT NULL;
ALTER TABLE step ADD COLUMN documentationMimeType VARCHAR(150)  DEFAULT NULL;
ALTER TABLE step ADD COLUMN documentationFileName VARCHAR(150)  DEFAULT NULL;


-- 2.5.1 add step status for start and stop
ALTER TABLE step ADD COLUMN state Integer Default 0 NOT NULL;

/* SPAGIC-464 */
ALTER TABLE relevant_data DROP FOREIGN KEY relevant_data_ibfk_1;
ALTER TABLE relevant_data DROP INDEX id_log;
ALTER TABLE relevant_data DROP COLUMN id_log;

/* SPAGIC-106*/
ALTER TABLE process_instance ADD COLUMN server VARCHAR(50);

/* SPAGIC 3 */
INSERT INTO type_service (id_ts, value) VALUES(7, 'OSGI-CONNECTOR');
INSERT INTO type_service (id_ts, value) VALUES(8, 'OSGI-SERVICE');

CREATE SEQUENCE spagic.seq_component;
CREATE SEQUENCE spagic.seq_service_instance;
CREATE SEQUENCE spagic.seq_property_service_instance;

Create table component (
	id_component Bigint NOT NULL,
	id_ts Bigint NOT NULL,
	name Varchar(100) NOT NULL,
	state Int NOT NULL,
 Primary Key (id_component)) Without Oids;

Create table service (
	id_service Varchar(300) NOT NULL,
	id_component Bigint NOT NULL,
	state Int,
	monitor_enabled Bool,
	startdate Timestamp NOT NULL,
	enddate Timestamp,
 Primary Key (id_service)) Without Oids;

Create table service_instance (
	id_service_instance Bigint NOT NULL,
	id_target_service_instance Bigint,
	id_service Varchar(300) NOT NULL,
	message_id Varchar(300) NOT NULL,
	correlation_id Varchar(300) NOT NULL,
	request Text,
	response Text,
	state Int NOT NULL,
	startdate Timestamp NOT NULL,
	enddate Timestamp,
 Primary Key (id_service_instance)) Without Oids;

Create table property_service (
	id_service Varchar(50) NOT NULL,
	id_property Bigint NOT NULL,
 Primary Key (id_service,id_property)) Without Oids;

Create table property_component (
	id_property Bigint NOT NULL,
	id_component Bigint NOT NULL,
 Primary Key (id_property,id_component)) Without Oids;

Create table property_service_instance (
	id_property Bigint NOT NULL,
	code Varchar(100) NOT NULL,
	value Varchar(500) NOT NULL,
	id_service_instance Bigint NOT NULL,
	UNIQUE (id_property),
 Primary Key (id_property)) Without Oids;
 
Create Index idx_service_instance_01 ON service_instance using btree (message_id);
Create Index idx_service_instance_02 ON service_instance using btree (correlation_id);

Alter table property_service add Foreign Key (id_property) references property (id_property) on delete  restrict on update  restrict;
Alter table property_component add Foreign Key (id_property) references property (id_property) on delete  restrict on update  restrict;
Alter table component add Foreign Key (id_ts) references type_service (id_ts) on delete  restrict on update  restrict;
Alter table service add Foreign Key (id_component) references component (id_component) on delete  restrict on update  restrict;
Alter table property_component add Foreign Key (id_component) references component (id_component) on delete  restrict on update  restrict;
Alter table service_instance add Foreign Key (id_service) references service (id_service) on delete  restrict on update  restrict;
Alter table property_service add Foreign Key (id_service) references service (id_service) on delete  restrict on update  restrict;
Alter table service_instance add Foreign Key (id_target_service_instance) references service_instance (id_service_instance) on delete  restrict on update  restrict;
Alter table property_service_instance add Foreign Key (id_service_instance) references service_instance (id_service_instance) on delete  restrict on update  restrict;

Create table schema_metadata (
	id_metadata Varchar(100) NOT NULL,
	data_schema Text NOT NULL,
 Primary Key (id_metadata)) Without Oids;
 
 /*SPAGIC 3.0 */
Alter table rule add COLUMN variable_name Varchar(200);

/* Restart management */
Alter table process_instance add COLUMN restarted Boolean Default false;

/* SPAGIC-690 */
Alter table transition_instance add attachment_id Text default NULL;
Alter table transition_instance drop COLUMN transition_data;
Alter table transition_instance drop COLUMN transition_data_out;
Alter table transition_instance drop COLUMN attachment_out;

------ Differences with Spagic 3.0.0 ------

Alter table service_instance add error Text DEFAULT NULL;

------ Differences with Spagic 3.1.0 ------

Create table attachments (
  id                   Varchar(200) NOT NULL,
  id_process_instance  Bigint,
  attachment_data      bytea NOT NULL,
  attachment_name	   Varchar(100),
  content_type         Varchar(50) NOT NULL,
  primary key (id)
); 

-- Differences with Spagic 3.2.0

-- New Table to handle Mutex
  
 CREATE TABLE lock_app (
	proc_name Varchar(255) NOT NULL,
	operation_type Varchar(255) NOT NULL,
	id_proc Varchar(255) NOT NULL,
	dt_last_mod Timestamp NOT NULL,
 Primary Key (proc_name,operation_type)) 
	WITHOUT Oids;

-- New Table to handle Node


 CREATE TABLE spagic_node (
	id_node Varchar(255) NOT NULL,
	ip_address Varchar(255),
	host Varchar(255),
	admin_port Int,
	ssl_port Int,
 Primary Key (id_node)) 
 WITHOUT Oids;

-- Association Between Service And Node


 CREATE TABLE service_node (
	id_service Varchar(300) NOT NULL,
	id_node Varchar(255) NOT NULL,
 Primary Key (id_service,id_node)) 
 WITHOUT Oids;


Alter table service_node add Foreign Key (id_service) references service (id_service) on delete  restrict on update  restrict;
Alter table service_node add Foreign Key (id_node) references spagic_node (id_node) on delete  restrict on update  restrict;


ALTER TABLE service_instance ADD COLUMN execution_node VARCHAR(255) NOT NULL DEFAULT 'node1' ;

create unique index uk_host_admin_port on spagic_node (host, admin_port);
create unique index uk_host_ssl_port on spagic_node (host, ssl_port);

Create Index fk_vi_pi ON variable_instance using btree (id_process_instance);

-- Differences with Spagic 3.3.0

-- usage of InVm monitoring
INSERT INTO COMPONENT(ID_TS, NAME, STATE) VALUES(7,'InVM',0);
INSERT INTO SERVICE(ID_SERVICE, ID_COMPONENT, STATE, MONITOR_ENABLED, STARTDATE, ENDDATE) VALUES ('InVM', (select ID_COMPONENT FROM COMPONENT WHERE NAME ='InVM'), 0, 1, CURRENT_TIMESTAMP, null);

alter table service_instance add column username VARCHAR(255) DEFAULT null;
alter table service_instance add column operation VARCHAR(255) DEFAULT null;
alter table service_instance add column id_process_instance Bigint;
ALTER TABLE process_instance ADD COLUMN actor_name VARCHAR(100) DEFAULT NULL;

ALTER table process_diagram ADD coords_info Text;
ALTER table process_diagram ADD graph bytea;

-- Application Field on service instance
ALTER TABLE service_instance ADD COLUMN application VARCHAR(255) DEFAULT NULL;

-- Additional flag "warning" on process instance state
ALTER TABLE process_instance ADD COLUMN warning_enabled Boolean Default false;



-- SPAGIC 3.5 FORM UPDATES ( ADDED IS PDF FIELD )
ALTER TABLE schema_metadata ADD COLUMN is_pdf Integer Default 0 NOT NULL;

CREATE SEQUENCE spagic.seq_process_configuration;

Create table process_configuration (
	id_process_conf Integer NOT NULL,
	id_process Integer NOT NULL ,
	logical_id Varchar(200) NOT NULL,
	is_default Integer NOT NULL,
	conf_data Text NOT NULL,
 primary key (id_process_conf));
 
Alter table process_configuration add Foreign Key (id_process) references process (id_process) on delete  restrict on update  restrict;

create index idx_service_instance_03 ON service_instance (id_service, state);

-- ENHANCE THE QUERIES BY STEP NAME AND STEP NAME AND ID_PROCESS
create index idx_step_01 ON step (name);
create index idx_step_02 ON step (name, id_process);
