/*
Created		07/11/2006
Modified	12/02/2009
Project		Spagic
Model		MetaDB 
Company		Engineering
Author		Spagic Team
Version		3.0.0, 2.6.0
Database	Oracle 9i 
*/


-- Create Tables section


Create table process_instance (
	id_process_instance Integer NOT NULL ,
	id_parent_process_instance Integer,
	id_process Integer NOT NULL ,
	startdate Timestamp NOT NULL ,
	enddate Timestamp NOT NULL ,
	executiontime Integer,
	state Integer NOT NULL ,
	id_correlation Varchar2 (100) NOT NULL ,
	id_pool Integer,
	id_process_execution_state Integer Default 0 NOT NULL ,
	server Varchar2 (50),
	restarted Char (1) Default 0,
	warning_enabled Char (1) Default 0,
	actor_name Varchar2(100),
primary key (id_process_instance) 
) 
;

Create table transition_instance (
	id_journal Integer NOT NULL ,
	id_step_source Integer,
	id_step_target Integer NOT NULL ,
	id_process_instance Integer NOT NULL ,
	id_message Varchar2 (100) NOT NULL ,
	transition_start Timestamp NOT NULL ,
	transition_end Timestamp,
	transition_execution_time Integer,
	id_ext_process_instance Integer,
	id_state Integer NOT NULL ,
	attachment_id Varchar2 (500),
	attachment_in Integer DEFAULT 0,
	runtime_warning Char(1) DEFAULT 0,
primary key (id_journal) 
) 
;

Create table log_journal (
	id_log Integer NOT NULL ,
	id_state Integer NOT NULL ,
	id_journal Integer NOT NULL ,
	name Varchar2 (200),
	log Clob NOT NULL ,
	datelog Timestamp NOT NULL ,
	id_log_journal_type Integer Default 0 NOT NULL ,
primary key (id_log) 
) 
;

Create table property (
	id_property Integer NOT NULL ,
	code Varchar2 (200) NOT NULL ,
	value Varchar2 (4000) NOT NULL ,
primary key (id_property) 
) 
;

Create table property_process (
	id_property Integer NOT NULL ,
	id_process Integer NOT NULL ,
primary key (id_property,id_process) 
) 
;

Create table process (
	id_process Integer NOT NULL ,
	process_version Varchar2 (20) NOT NULL ,
	namespace Varchar2 (200) NOT NULL ,
	name Varchar2 (200) NOT NULL ,
	description Varchar2 (200),
	id_ts Integer NOT NULL ,
	uddi_org Varchar2 (200),
	uddi_name Varchar2 (200),
	startdate Timestamp NOT NULL ,
	enddate Timestamp,
	state Integer NOT NULL ,
	monitor_enabled Char (1) Default 1,
	id_monitoring_type Integer Default 0 NOT NULL ,
primary key (id_process) 
) 
;

Create table step (
	id_step Integer NOT NULL ,
	namespace Varchar2 (200) NOT NULL ,
	name Varchar2 (200) NOT NULL ,
	description Varchar2 (200),
	is_start Char (1) NOT NULL ,
	is_end Char (1) NOT NULL ,
	id_process Integer NOT NULL ,
	process_version Char (20) NOT NULL ,
	has_conditional_exit Char(1),
	documentation BLOB  DEFAULT NULL,
	documentationMimeType VARCHAR2(150)  DEFAULT NULL,
	documentationFileName VARCHAR2(150)  DEFAULT NULL,
	state Integer Default 0 NOT NULL,
primary key (id_step) 
) 
;

Create table type_service (
	id_ts Integer NOT NULL ,
	value Varchar2 (45) NOT NULL ,
primary key (id_ts) 
) 
;

Create table property_step (
	id_step Integer NOT NULL ,
	id_property Integer NOT NULL ,
primary key (id_step,id_property) 
) 
;

Create table transition_state (
	id_state Integer NOT NULL ,
	state_value Varchar2 (100),
primary key (id_state) 
) 
;

Create table process_diagram (
	svg_diag Clob,
	id_process Integer NOT NULL ,
	coords_info Clob,
	graph Blob,
primary key (id_process) 
) 
;


Create table relevant_data (
	id_relevant_data Integer NOT NULL ,
	id_attribute Integer NOT NULL ,
	value Varchar2 (500) NOT NULL ,
primary key (id_relevant_data) 
) 
;


Create table catalog (
	id_catalog Integer NOT NULL ,
	name Varchar2 (200) NOT NULL ,
	visible Char (1),
	mandatory Char (1),
primary key (id_catalog) 
) 
;

Create table attribute (
	id_attribute Integer NOT NULL ,
	name Varchar2 (50) NOT NULL  UNIQUE ,
	type Integer NOT NULL ,
primary key (id_attribute) 
) 
;

Create table transition (
	id_step_source Integer NOT NULL ,
	id_step_target Integer NOT NULL ,
	id_process Integer NOT NULL ,
primary key (id_step_source,id_step_target) 
) 
;

Create table pool_type (
	id_pool_type Integer NOT NULL,
	name Varchar2 (200) NOT NULL ,
	description Varchar2 (200) NOT NULL ,
	pool_type_version Char (20) NOT NULL ,
primary key (id_pool_type) 
) 
;

Create table pool_instance (
	id_pool Integer NOT NULL ,
	id_pool_type Integer NOT NULL ,
	unique_pool_instance_key Varchar2 (1000) NOT NULL ,
primary key (id_pool) 
) 
;

Create table pool_type_process (
	id_process Integer NOT NULL ,
	order_criteria Integer NOT NULL ,
	id_pool_type Integer NOT NULL ,
primary key (id_process,id_pool_type) 
) 
;

Create table relevant_data_process_instance (
	id_process_instance Integer NOT NULL ,
	id_relevant_data Integer NOT NULL ,
primary key (id_process_instance,id_relevant_data) 
) 
;

Create table xml_namespace (
	id_namespace Integer NOT NULL ,
	prefix Varchar2 (50) NOT NULL ,
	value Varchar2 (200) NOT NULL ,
primary key (id_namespace) 
) 
;

Create table attribute_pool (
	id_attribute Integer NOT NULL ,
	id_pool_type Integer NOT NULL ,
primary key (id_attribute,id_pool_type) 
) 
;

Create table attribute_catalog (
	id_attribute Integer NOT NULL ,
	id_catalog Integer NOT NULL ,
	visible Char (1),
	order_criteria Integer,
primary key (id_attribute,id_catalog) 
) 
;


Create table search_query (
   id_query Integer NOT NULL ,
   label Varchar(100) NOT NULL UNIQUE,
   query Clob NOT NULL,
Primary Key (id_query)
)
;

Create table sub_process (
	id_process Integer NOT NULL ,
	id_step Integer NOT NULL,
	id_subprocess Integer NOT NULL ,
	descr Varchar2 (50),
primary key (id_process,id_step, id_subprocess) 
) 
;

Create table process_execution_state (
	id_process_execution_state Integer NOT NULL ,
	state_value Varchar2 (100),
primary key (id_process_execution_state) 
) 
;

Create table monitoring_type (
	id_monitoring_type Integer NOT NULL ,
	type_value Varchar2 (100),
primary key (id_monitoring_type) 
) 
;

Create table temporary_process_instance (
	id_process_instance Integer NOT NULL,
primary key (id_process_instance));

Create table variable_instance (
	id_variable_instance Integer NOT NULL ,
	id_variable_definition Integer,
	name Varchar2 (100) NOT NULL ,
	value Clob,
	id_process_instance Integer NOT NULL ,
primary key (id_variable_instance) 
) 
;

Create table variable_definition (
	id_variable_definition Integer NOT NULL ,
	name Varchar2 (100) NOT NULL ,
	varSchema Clob,
	id_process Integer NOT NULL ,
primary key (id_variable_definition) 
) 
;

Create table log_journal_type (
	id_log_journal_type Integer NOT NULL ,
	type_journal Varchar2 (50) NOT NULL 
) 
;

Create table component (
	id_component Integer NOT NULL ,
	id_ts Integer NOT NULL ,
	name Varchar2 (100) NOT NULL ,
	state Integer NOT NULL ,
primary key (id_component) 
) 
;

Create table service (
	id_service Varchar2 (300) NOT NULL ,
	id_component Integer NOT NULL ,
	state Integer,
	monitor_enabled Char (1),
	startdate Timestamp(0) NOT NULL ,
	enddate Timestamp(0),
primary key (id_service) 
) 
;

Create table service_instance (
	id_service_instance Integer NOT NULL ,
	id_target_service_instance Integer,
	id_service Varchar2 (300) NOT NULL ,
	message_id Varchar2 (300) NOT NULL ,
	correlation_id Varchar2 (300) NOT NULL ,
	request Clob,
	response Clob,
	error Clob,
	state Integer NOT NULL ,
	startdate Timestamp(0) NOT NULL ,
	enddate Timestamp(0),
	execution_node Varchar2 (255) DEFAULT 'node1' NOT NULL ,
	username Varchar2 (255) Default NULL,
	operation Varchar2 (255) Default NULL,
	application Varchar2 (255),
	id_process_instance Integer,
primary key (id_service_instance) 
) 
;

Create table property_service (
	id_service Varchar2 (255) NOT NULL ,
	id_property Integer NOT NULL ,
primary key (id_service,id_property) 
) 
;

Create table property_component (
	id_property Integer NOT NULL ,
	id_component Integer NOT NULL ,
primary key (id_property,id_component) 
) 
;

Create table property_service_instance (
	   -- Saint - Tue Nov 17 10:27:31 2009
	   -- 
	   -- removed the error causing UNIQUE (at least in Oracle 11)
	id_property Integer NOT NULL, --  UNIQUE ,
	   --  </Saint>
	code Varchar2 (100) NOT NULL ,
	value Varchar2 (500) NOT NULL ,
	id_service_instance Integer NOT NULL ,
primary key (id_property) 
) 
;

--Create table schema_metadata (
--	id_metadata Varchar(100) NOT NULL,
--	data_schema Clob NOT NULL,
--	is_pdf Integer,
-- Primary Key (id_metadata));
 
Create table attachments (
  id                   Varchar2(200) NOT NULL,
  id_process_instance  Integer,
  attachment_data      Blob NOT NULL,
  attachment_name	   Varchar2(100),
  content_type         Varchar2(50) NOT NULL,
  primary key (id)
); 
 
Create table exchange_store (
  id               Integer NOT NULL,
  process_id      Integer NOT NULL,
  data             Blob,
  primary key (id)
);




 -- Create View section
CREATE VIEW bc_input_messages AS
  SELECT s.id_step AS id_binding_component, COUNT(logJournal.id_log) AS input_messages 
	FROM step s, process p, transition_instance t, log_journal logJournal,property_step propertyStep, property pty  	
	WHERE s.id_process = p.id_process AND s.id_process = (SELECT max(pSub.id_process) AS idProcess FROM process pSub WHERE pSub.name=p.name)				
				AND s.id_step = t.id_step_source
				AND t.id_journal = logJournal.id_journal 				
				AND s.id_step=propertyStep.id_step 				
				AND propertyStep.id_property=pty.id_property
				AND pty.code='im.servicebindingname'
				AND ((s.is_start = 1 AND logJournal.id_state IN (0, 3)) 
						OR (pty.value IN ('JBI-WebServicePipeline','JBI-TCPIP-Pipeline','JBI-CXF-Pipeline') AND logJournal.id_state=3 )
						) 				
	group by s.id_step;

CREATE VIEW bc_output_messages AS
  SELECT s.id_step AS id_binding_component, COUNT(logJournal.id_log) AS output_messages 
	FROM step s, process p, transition_instance t, log_journal logJournal,property_step propertyStep, property pty  	
	WHERE s.id_process = p.id_process AND s.id_process = (SELECT max(pSub.id_process) AS idProcess FROM process pSub WHERE pSub.name=p.name)								
				AND t.id_journal = logJournal.id_journal 				
				AND s.id_step=propertyStep.id_step 				
				AND propertyStep.id_property=pty.id_property
				AND pty.code='im.servicebindingname'
				AND ((s.is_end = 1 AND logJournal.id_state=1 AND s.id_step = t.id_step_target)
						OR (pty.value IN ('JBI-WebServicePipeline','JBI-TCPIP-Pipeline','JBI-CXF-Pipeline') AND s.id_step = t.id_step_source AND logJournal.id_state=4 )
            OR (s.is_start = 1 AND logJournal.id_state = 4 AND s.id_step = t.id_step_source)						
						) 				
	group by s.id_step;
	
CREATE VIEW bc_error_messages AS
  SELECT s.id_step AS id_binding_component, COUNT(logJournal.id_log) AS error_messages 
	FROM step s, process p, transition_instance t, log_journal logJournal,property_step propertyStep, property pty  	
	WHERE s.id_process = p.id_process AND s.id_process = (SELECT max(pSub.id_process) AS idProcess FROM process pSub WHERE pSub.name=p.name)								
				AND t.id_journal = logJournal.id_journal 				
				AND s.id_step=propertyStep.id_step 				
				AND propertyStep.id_property=pty.id_property
				AND pty.code='im.servicebindingname'
				AND ((s.is_start = 1 AND s.id_step = t.id_step_source AND logJournal.id_state IN (2, 5))
						OR(s.is_end = 1 AND s.id_step = t.id_step_target AND logJournal.id_state IN (2, 5))
						OR (pty.value IN ('JBI-WebServicePipeline','JBI-TCPIP-Pipeline','JBI-CXF-Pipeline') AND logJournal.id_state=5
								AND (s.id_step = t.id_step_target OR s.id_step = t.id_step_source))
						) 				
	group by s.id_step;

-- Create Indexes section

Create Index fk_p_sa ON process_instance (id_process);
Create Index fk_jip_p ON transition_instance (id_process_instance);
Create Index fk_sa_ts ON process (id_ts);

Create Index idx_process_instance_01 ON process_instance (id_correlation);
/* Cannot be unique due to restarted instances */
Create Index idx_transition_instance_01 ON transition_instance (id_message);
Create Index idx_log_journal_01 ON log_journal (id_journal);
Create Index idx_log_journal_02 ON log_journal (id_state);

Create Index idx_service_instance_01 ON service_instance (message_id);
Create Index idx_service_instance_02 ON service_instance (correlation_id);

Create Index fk_vi_pi ON variable_instance (id_process_instance);

create index idx_service_instance_03 ON service_instance (id_service, state);

 -- ENHANCE THE QUERIES BY STEP NAME AND STEP NAME AND ID_PROCESS
create index idx_step_01 ON step (name);
create index idx_step_02 ON step (name, id_process);

-- Create Foreign keys section

Alter table transition_instance add  foreign key (id_process_instance) references process_instance (id_process_instance);
Alter table relevant_data_process_instance add  foreign key (id_process_instance) references process_instance (id_process_instance);
Alter table log_journal add  foreign key (id_journal) references transition_instance (id_journal);
Alter table process_instance add  foreign key (id_parent_process_instance) references process_instance (id_process_instance);
Alter table property_process add  foreign key (id_property) references property (id_property);
Alter table property_step add  foreign key (id_property) references property (id_property);
Alter table process_instance add  foreign key (id_process) references process (id_process);
Alter table property_process add  foreign key (id_process) references process (id_process);
Alter table step add  foreign key (id_process) references process (id_process);
Alter table process_diagram add  foreign key (id_process) references process (id_process);
Alter table transition add  foreign key (id_process) references process (id_process);
Alter table pool_type_process add  foreign key (id_process) references process (id_process);
Alter table transition_instance add  foreign key (id_step_source) references step (id_step);
Alter table property_step add  foreign key (id_step) references step (id_step);
Alter table transition_instance add  foreign key (id_step_target) references step (id_step);
Alter table transition add  foreign key (id_step_source) references step (id_step);
Alter table transition add  foreign key (id_step_target) references step (id_step);
Alter table process add  foreign key (id_ts) references type_service (id_ts);
Alter table transition_instance add  foreign key (id_state) references transition_state (id_state);
Alter table log_journal add  foreign key (id_state) references transition_state (id_state);
Alter table relevant_data_process_instance add  foreign key (id_relevant_data) references relevant_data (id_relevant_data);
Alter table attribute_catalog add  foreign key (id_catalog) references catalog (id_catalog);
Alter table relevant_data add  foreign key (id_attribute) references attribute (id_attribute);
Alter table attribute_pool add  foreign key (id_attribute) references attribute (id_attribute);
Alter table attribute_catalog add  foreign key (id_attribute) references attribute (id_attribute);

Alter table pool_instance add  foreign key (id_pool_type) references pool_type (id_pool_type);
Alter table pool_type_process add  foreign key (id_pool_type) references pool_type (id_pool_type);
Alter table attribute_pool add  foreign key (id_pool_type) references pool_type (id_pool_type);
Alter table process_instance add  foreign key (id_pool) references pool_instance (id_pool);

ALTER TABLE sub_process add foreign key (id_process) references process (id_process);
ALTER TABLE sub_process add foreign key (id_subprocess) references process (id_process);
ALTER TABLE sub_process add foreign key (id_step) references step (id_step);

Alter table process_instance add  foreign key (id_process_execution_state) references process_execution_state (id_process_execution_state);
Alter table process add  foreign key (id_monitoring_type) references monitoring_type (id_monitoring_type);
Alter table variable_instance add  foreign key (id_process_instance) references process_instance (id_process_instance);
Alter table variable_definition add  foreign key (id_process) references process (id_process);
Alter table variable_instance add  foreign key (id_variable_definition) references variable_definition (id_variable_definition);

Alter table property_service add Foreign Key (id_property) references property (id_property);
Alter table property_component add Foreign Key (id_property) references property (id_property);
Alter table component add Foreign Key (id_ts) references type_service (id_ts);
Alter table service add Foreign Key (id_component) references component (id_component);
Alter table property_component add Foreign Key (id_component) references component (id_component);
Alter table service_instance add Foreign Key (id_service) references service (id_service);
Alter table property_service add Foreign Key (id_service) references service (id_service);
Alter table service_instance add Foreign Key (id_target_service_instance) references service_instance (id_service_instance);
Alter table property_service_instance add Foreign Key (id_service_instance) references service_instance (id_service_instance);

-- Create Sequences section

CREATE SEQUENCE seq_process_instance;
CREATE SEQUENCE seq_transition_instance;
CREATE SEQUENCE seq_log_journal;
CREATE SEQUENCE seq_property;
CREATE SEQUENCE seq_property_process;
CREATE SEQUENCE seq_process;
CREATE SEQUENCE seq_step;
CREATE SEQUENCE seq_property_step;
CREATE SEQUENCE seq_transition_state;
CREATE SEQUENCE seq_process_diagram;
CREATE SEQUENCE seq_relevant_data;

CREATE SEQUENCE seq_catalog;
CREATE SEQUENCE seq_attribute;
CREATE SEQUENCE seq_transition;
CREATE SEQUENCE seq_pool_type;
CREATE SEQUENCE seq_pool_instance;
CREATE SEQUENCE seq_pool_type_process;
CREATE SEQUENCE seq_xml_namespace;
CREATE SEQUENCE seq_attribute_pool;
CREATE SEQUENCE seq_attribute_catalog;

CREATE SEQUENCE seq_search_query;
CREATE SEQUENCE seq_variable_instance;
CREATE SEQUENCE seq_variable_definition;

CREATE SEQUENCE seq_component;
CREATE SEQUENCE seq_service_instance;
CREATE SEQUENCE seq_property_service_instance;

-- Saint - Tue Feb 23 11:30:30 2010
-- 
-- a new sequence for the attachment store
CREATE SEQUENCE seq_exchange_store;
--  </Saint>

CREATE SEQUENCE seq_type_service;

CREATE SEQUENCE seq_process_configuration;

-- Differences with Spagic 3.2.0

-- New Table to handle Mutex
 CREATE TABLE lock_app (
	proc_name Varchar(255) NOT NULL,
	operation_type Varchar(255) NOT NULL,
	id_proc Varchar(255) NOT NULL,
	dt_last_mod Timestamp NOT NULL,
 Primary Key (proc_name,operation_type));

-- New Table to handle Node
CREATE TABLE spagic_node (
	id_node Varchar(255) NOT NULL,
	ip_address Varchar(255),
	host Varchar(255),
	admin_port Int,
	ssl_port Int,
 Primary Key (id_node)) ;

-- Association Between Service And Node
CREATE TABLE service_node (
	id_service Varchar(300) NOT NULL,
	id_node Varchar(255) NOT NULL,
 Primary Key (id_service,id_node));

CREATE TABLE form_data (
	id_process_instance Integer NOT NULL,
	id_step Integer NOT NULL,
	fdf Blob,
	pdfName Varchar(50),	
 Primary Key (id_process_instance, id_step));

 CREATE TABLE  spagic_token (
	token Varchar(255) NOT NULL,
	dt_expiration Timestamp NOT NULL,
 primary key (token));
 
Alter table service_node add Foreign Key (id_service) references service (id_service);
Alter table service_node add Foreign Key (id_node) references spagic_node (id_node) ;

create unique index uk_host_admin_port on spagic_node (host, admin_port);
create unique index uk_host_ssl_port on spagic_node (host, ssl_port);


Alter table form_data add Foreign Key (id_process_instance) references process_instance (id_process_instance);
Alter table form_data add Foreign Key (id_step) references step (id_step);


Create table process_configuration (
	id_process_conf Integer NOT NULL,
	id_process Integer NOT NULL ,
	logical_id Varchar(200) NOT NULL,
	is_default Integer NOT NULL,
	conf_data Clob NOT NULL,
 primary key (id_process_conf));
 
 Alter table process_configuration add Foreign Key (id_process) references process (id_process);
 