/*
Created		07/11/2006
Modified	12/02/2009
Project		Spagic
Model		MetaDB 
Company		Engineering
Author		Spagic Team
Version		3.0.0, 2.6.0
Database	PostgreSQL 8.1 
*/


/* Create Sequences */

CREATE SEQUENCE spagic.seq_process_instance;

CREATE SEQUENCE spagic.seq_transition_instance;

CREATE SEQUENCE spagic.seq_log_journal;

CREATE SEQUENCE spagic.seq_property;

CREATE SEQUENCE spagic.seq_property_process;

CREATE SEQUENCE spagic.seq_process;

CREATE SEQUENCE spagic.seq_step;

CREATE SEQUENCE spagic.seq_property_step;

CREATE SEQUENCE spagic.seq_transition_state;

CREATE SEQUENCE spagic.seq_process_diagram;

CREATE SEQUENCE spagic.seq_relevant_data;

CREATE SEQUENCE spagic.seq_catalog;

CREATE SEQUENCE spagic.seq_attribute;

CREATE SEQUENCE spagic.seq_transition;

CREATE SEQUENCE spagic.seq_pool_type;

CREATE SEQUENCE spagic.seq_pool_instance;

CREATE SEQUENCE spagic.seq_pool_type_process;

CREATE SEQUENCE spagic.seq_xml_namespace;

CREATE SEQUENCE spagic.seq_attribute_pool;

CREATE SEQUENCE spagic.seq_attribute_catalog;

CREATE SEQUENCE spagic.seq_search_query;

CREATE SEQUENCE spagic.seq_component;
CREATE SEQUENCE spagic.seq_service_instance;
CREATE SEQUENCE spagic.seq_property_service_instance;

CREATE SEQUENCE spagic.seq_variable_instance;
CREATE SEQUENCE spagic.seq_variable_definition;

CREATE SEQUENCE spagic.seq_type_service;

-- Saint - Tue Feb 23 11:25:59 2010
-- 
-- a new sequence for the attachment store
CREATE SEQUENCE spagic.seq_exchange_store;

CREATE SEQUENCE spagic.seq_process_configuration;
--  </Saint>

/* Create Tables */

Create table process_instance
(
	id_process_instance Bigint NOT NULL,
	id_parent_process_instance Bigint,
	id_process Bigint NOT NULL,
	startdate Timestamp NOT NULL,
	enddate Timestamp NOT NULL,
	executiontime Bigint,
	state Integer NOT NULL,
	id_correlation Varchar(100) NOT NULL,
	id_pool Bigint,
	id_process_execution_state Bigint NOT NULL Default 0,
	server Varchar(50),
	restarted Boolean Default false,
	warning_enabled Boolean Default false,
	actor_name Varchar(100),
 primary key (id_process_instance)
) Without Oids;


Create table transition_instance
(
	id_journal Bigint NOT NULL,
	id_step_source Bigint,
	id_step_target Bigint NOT NULL,
	id_process_instance Bigint NOT NULL,
	id_message Varchar(100) NOT NULL,
	transition_start Timestamp NOT NULL,
	transition_end Timestamp,
	transition_execution_time Bigint,
	id_ext_process_instance Bigint,
	id_state Integer NOT NULL,
	attachment_id Text,
	attachment_in Integer Default 0,
	runtime_warning Boolean DEFAULT FALSE,
 primary key (id_journal)
) Without Oids;


Create table log_journal
(
	id_log Bigint NOT NULL,
	id_state Integer NOT NULL,
	id_journal Bigint NOT NULL,
	name Varchar(200),
	log Text NOT NULL,
	datelog Timestamp NOT NULL,
	id_log_journal_type Bigint NOT NULL DEFAULT 0,
 primary key (id_log)
) Without Oids;

Create table log_journal_type (
	id_log_journal_type Bigint NOT NULL,
	type_journal Varchar(50) NOT NULL,
 primary key (id_log_journal_type)
) Without Oids;

Create table property
(
	id_property Bigint NOT NULL,
	code Varchar(200) NOT NULL,
	value Varchar(10000) NOT NULL,
 primary key (id_property)
) Without Oids;


Create table property_process
(
	id_property Bigint NOT NULL,
	id_process Bigint NOT NULL,
 primary key (id_property,id_process)
) Without Oids;


Create table process
(
	id_process Bigint NOT NULL,
	process_version Varchar(20) NOT NULL,
	namespace Varchar(200) NOT NULL,
	name Varchar(200) NOT NULL,
	description Varchar(200),
	id_ts Bigint NOT NULL,
	uddi_org Varchar(200),
	uddi_name Varchar(200),
	startdate Timestamp NOT NULL,
	enddate Timestamp,
	state Integer NOT NULL,
	monitor_enabled Boolean Default true,
	id_monitoring_type Bigint NOT NULL Default 0,
 primary key (id_process)
) Without Oids;


Create table step
(
	id_step Bigint NOT NULL,
	namespace Varchar(200) NOT NULL,
	name Varchar(200) NOT NULL,
	description Varchar(200),
	is_start Boolean NOT NULL,
	is_end Boolean NOT NULL,
	id_process Bigint NOT NULL,
	process_version Char(20) NOT NULL,
	has_conditional_exit Boolean,
	documentation bytea DEFAULT NULL,
	documentationMimeType VARCHAR(150)  DEFAULT NULL,
	documentationFileName VARCHAR(150)  DEFAULT NULL,
	state Integer Default 0 NOT NULL,
 primary key (id_step)
) Without Oids;


Create table type_service
(
	id_ts Bigint NOT NULL,
	value Varchar(45) NOT NULL,
 primary key (id_ts)
) Without Oids;


Create table property_step
(
	id_step Bigint NOT NULL,
	id_property Bigint NOT NULL,
 primary key (id_step,id_property)
) Without Oids;


Create table transition_state
(
	id_state Integer NOT NULL,
	state_value Varchar(100),
 primary key (id_state)
) Without Oids;


Create table process_diagram
(
	svg_diag Text,
	id_process Bigint NOT NULL,
	coords_info Text,
	graph bytea,
 primary key (id_process)
) Without Oids;


Create table relevant_data
(
	id_relevant_data Bigint NOT NULL,
	id_attribute Bigint NOT NULL,
	value Varchar(500) NOT NULL,
 primary key (id_relevant_data)
) Without Oids;


Create table catalog
(
	id_catalog Bigint NOT NULL,
	name Varchar(200) NOT NULL,
	visible Boolean,
	mandatory Boolean,
 primary key (id_catalog)
) Without Oids;


Create table attribute
(
	id_attribute Bigint NOT NULL,
	name Varchar(50) NOT NULL UNIQUE,
	type Integer NOT NULL,
 primary key (id_attribute)
) Without Oids;


Create table transition
(
	id_step_source Bigint NOT NULL,
	id_step_target Bigint NOT NULL,
	id_process Bigint NOT NULL,
 primary key (id_step_source,id_step_target)
) Without Oids;


Create table pool_type
(
	id_pool_type Bigint NOT NULL,
	name Varchar(200) NOT NULL,
	description Varchar(200) NOT NULL,
	pool_type_version Char(20) NOT NULL,
 primary key (id_pool_type)
) Without Oids;


Create table pool_instance
(
	id_pool Bigint NOT NULL,
	id_pool_type Bigint NOT NULL,
	unique_pool_instance_key Varchar(1000) NOT NULL,
 primary key (id_pool)
) Without Oids;


Create table pool_type_process
(
	id_process Bigint NOT NULL,
	order_criteria Integer NOT NULL,
	id_pool_type Bigint NOT NULL,
 primary key (id_process,id_pool_type)
) Without Oids;


Create table relevant_data_process_instance
(
	id_process_instance Bigint NOT NULL,
	id_relevant_data Bigint NOT NULL,
 primary key (id_process_instance,id_relevant_data)
) Without Oids;


Create table xml_namespace
(
	id_namespace Bigint NOT NULL,
	prefix Varchar(50) NOT NULL,
	value Varchar(200) NOT NULL,
 primary key (id_namespace)
) Without Oids;


Create table attribute_pool
(
	id_attribute Bigint NOT NULL,
	id_pool_type Bigint NOT NULL,
 primary key (id_attribute,id_pool_type)
) Without Oids;


Create table attribute_catalog
(
	id_attribute Bigint NOT NULL,
	id_catalog Bigint NOT NULL,
	visible Boolean,
	order_criteria Integer,
 primary key (id_attribute,id_catalog)
) Without Oids;

Create table search_query (
   id_query Integer NOT NULL ,
   label Varchar(100) NOT NULL UNIQUE,
   query Text NOT NULL,
Primary Key (id_query)
) Without Oids;

Create table sub_process
(
	id_process Bigint NOT NULL,
	id_subprocess Bigint NOT NULL,
	descr Varchar(50),
 primary key (id_process,id_subprocess)
) Without Oids;

Create table process_execution_state
(
	id_process_execution_state Bigint NOT NULL,
	state_value Varchar(100),
 primary key (id_process_execution_state)
) Without Oids;

Create table monitoring_type
(
	id_monitoring_type Bigint NOT NULL,
	type_value Varchar(100),
 primary key (id_monitoring_type)
) Without Oids;

Create table temporary_process_instance (
	id_process_instance Bigint NOT NULL,
primary key (id_process_instance));

Create table variable_instance (
	id_variable_instance Bigint NOT NULL,
	id_variable_definition Bigint,
	name Varchar(100) NOT NULL,
	value Text,
	id_process_instance Bigint NOT NULL,
primary key (id_variable_instance)
) Without Oids;

Create table variable_definition (
	id_variable_definition Bigint NOT NULL,
	name Varchar(100) NOT NULL,
	varSchema Text,
	id_process Bigint NOT NULL,
 primary key (id_variable_definition)
) Without Oids;
 

Create table component (
	id_component Bigint NOT NULL,
	id_ts Bigint NOT NULL,
	name Varchar(100) NOT NULL,
	state Int NOT NULL,
 Primary Key (id_component)) Without Oids;

Create table service (
	id_service Varchar(300) NOT NULL,
	id_component Bigint NOT NULL,
	state Int,
	monitor_enabled Bool,
	startdate Timestamp NOT NULL,
	enddate Timestamp,
 Primary Key (id_service)) Without Oids;

Create table service_instance (
	id_service_instance Bigint NOT NULL,
	id_target_service_instance Bigint,
	id_service Varchar(300) NOT NULL,
	message_id Varchar(300) NOT NULL,
	correlation_id Varchar(300) NOT NULL,
	request Text,
	response Text,
	error Text,
	state Int NOT NULL,
	startdate Timestamp NOT NULL,
	enddate Timestamp,
	execution_node VARCHAR(255) NOT NULL DEFAULT 'node1',
	username VARCHAR(255),
	operation VARCHAR(255),
	application VARCHAR(255),
	id_process_instance Bigint,
 Primary Key (id_service_instance)) Without Oids;

Create table property_service (
	id_service Varchar(255) NOT NULL,
	id_property Bigint NOT NULL,
 Primary Key (id_service,id_property)) Without Oids;

Create table property_component (
	id_property Bigint NOT NULL,
	id_component Bigint NOT NULL,
 Primary Key (id_property,id_component)) Without Oids;

Create table property_service_instance (
	id_property Bigint NOT NULL,
	code Varchar(100) NOT NULL,
	value Varchar(500) NOT NULL,
	id_service_instance Bigint NOT NULL,
	UNIQUE (id_property),
 Primary Key (id_property)) Without Oids;
 
--Create table schema_metadata (
--	id_metadata Varchar(100) NOT NULL,
--	data_schema Text NOT NULL,
--	is_pdf Integer NOT NULL,
 --Primary Key (id_metadata)) Without Oids;
 
Create table attachments (
  id                   Varchar(200) NOT NULL,
  id_process_instance  Bigint ,
  attachment_data      bytea NOT NULL,
  attachment_name	   Varchar(100),
  content_type         Varchar(50) NOT NULL,
  primary key (id)
); 
 

Create table exchange_store (
  id               Bigint NOT NULL,
  process_id       Bigint NOT NULL,
  data             bytea,
  primary key (id)
);

 /* -- Create view section */
CREATE VIEW bc_input_messages AS
  SELECT s.id_step AS id_binding_component, COUNT(logJournal.id_log) AS input_messages 
	FROM step s, process p, transition_instance t, log_journal logJournal,property_step propertyStep, property pty  	
	WHERE s.id_process = p.id_process AND s.id_process = (SELECT max(pSub.id_process) AS idProcess FROM process pSub WHERE pSub.name=p.name)				
				AND s.id_step = t.id_step_source
				AND t.id_journal = logJournal.id_journal 				
				AND s.id_step=propertyStep.id_step 				
				AND propertyStep.id_property=pty.id_property
				AND pty.code='im.servicebindingname'
				AND ((s.is_start AND logJournal.id_state IN (0, 3)) 
						OR (pty.value IN ('JBI-WebServicePipeline','JBI-TCPIP-Pipeline','JBI-CXF-Pipeline') AND logJournal.id_state=3 )
						) 				
	group by s.id_step;

CREATE VIEW bc_output_messages AS
  SELECT s.id_step AS id_binding_component, COUNT(logJournal.id_log) AS output_messages 
	FROM step s, process p, transition_instance t, log_journal logJournal,property_step propertyStep, property pty  	
	WHERE s.id_process = p.id_process AND s.id_process = (SELECT max(pSub.id_process) AS idProcess FROM process pSub WHERE pSub.name=p.name)								
				AND t.id_journal = logJournal.id_journal 				
				AND s.id_step=propertyStep.id_step 				
				AND propertyStep.id_property=pty.id_property
				AND pty.code='im.servicebindingname'
				AND ((s.is_end AND logJournal.id_state=1 AND s.id_step = t.id_step_target)
						OR (pty.value IN ('JBI-WebServicePipeline','JBI-TCPIP-Pipeline','JBI-CXF-Pipeline') AND s.id_step = t.id_step_source AND logJournal.id_state=4 )
            OR (s.is_start AND logJournal.id_state = 4 AND s.id_step = t.id_step_source)						
						) 				
	group by s.id_step;
	
CREATE VIEW bc_error_messages AS
  SELECT s.id_step AS id_binding_component, COUNT(logJournal.id_log) AS error_messages 
	FROM step s, process p, transition_instance t, log_journal logJournal,property_step propertyStep, property pty  	
	WHERE s.id_process = p.id_process AND s.id_process = (SELECT max(pSub.id_process) AS idProcess FROM process pSub WHERE pSub.name=p.name)								
				AND t.id_journal = logJournal.id_journal 				
				AND s.id_step=propertyStep.id_step 				
				AND propertyStep.id_property=pty.id_property
				AND pty.code='im.servicebindingname'
				AND ((s.is_start AND s.id_step = t.id_step_source AND logJournal.id_state IN (2, 5))
						OR(s.is_end  AND s.id_step = t.id_step_target AND logJournal.id_state IN (2, 5))
						OR (pty.value IN ('JBI-WebServicePipeline','JBI-TCPIP-Pipeline','JBI-CXF-Pipeline') AND logJournal.id_state=5
								AND (s.id_step = t.id_step_target OR s.id_step = t.id_step_source))
						) 				
	group by s.id_step;


/* Create Indexes */
Create index fk_p_sa on process_instance using btree (id_process);
Create index fk_jip_p on transition_instance using btree (id_process_instance);
Create index fk_sa_ts on process using btree (id_ts);

Create index idx_process_instance_01 on process_instance using btree (id_correlation);
/* Cannot be unique due to restarted instances */
Create Index idx_transition_instance_01 ON transition_instance using btree (id_message);
Create Index idx_log_journal_01 ON log_journal using btree (id_journal);
Create Index idx_log_journal_02 ON log_journal using btree (id_state);

Create Index idx_service_instance_01 ON service_instance using btree (message_id);
Create Index idx_service_instance_02 ON service_instance using btree (correlation_id);

Create Index fk_vi_pi ON variable_instance using btree (id_process_instance);

create index idx_service_instance_03 ON service_instance (id_service, state);

-- ENHANCE THE QUERIES BY STEP NAME AND STEP NAME AND ID_PROCESS
create index idx_step_01 ON step (name);
create index idx_step_02 ON step (name, id_process);

/* Create Foreign Keys */

Alter table transition_instance add  foreign key (id_process_instance) references process_instance (id_process_instance) on update restrict on delete restrict;

Alter table relevant_data_process_instance add  foreign key (id_process_instance) references process_instance (id_process_instance) on update restrict on delete restrict;

Alter table process_instance add  foreign key (id_parent_process_instance) references process_instance (id_process_instance) on update restrict on delete restrict;

Alter table log_journal add  foreign key (id_journal) references transition_instance (id_journal) on update restrict on delete restrict;

Alter table property_process add  foreign key (id_property) references property (id_property) on update restrict on delete restrict;

Alter table property_step add  foreign key (id_property) references property (id_property) on update restrict on delete restrict;

Alter table process_instance add  foreign key (id_process) references process (id_process) on update restrict on delete restrict;

Alter table property_process add  foreign key (id_process) references process (id_process) on update restrict on delete restrict;

Alter table step add  foreign key (id_process) references process (id_process) on update restrict on delete restrict;

Alter table process_diagram add  foreign key (id_process) references process (id_process) on update restrict on delete restrict;

Alter table transition add  foreign key (id_process) references process (id_process) on update restrict on delete restrict;

Alter table pool_type_process add  foreign key (id_process) references process (id_process) on update restrict on delete restrict;

Alter table sub_process add  foreign key (id_process) references process (id_process) on update restrict on delete restrict;

Alter table sub_process add  foreign key (id_subprocess) references process (id_process) on update restrict on delete restrict;

Alter table transition_instance add  foreign key (id_step_source) references step (id_step) on update restrict on delete restrict;

Alter table property_step add  foreign key (id_step) references step (id_step) on update restrict on delete restrict;

Alter table transition_instance add  foreign key (id_step_target) references step (id_step) on update restrict on delete restrict;

Alter table transition add  foreign key (id_step_source) references step (id_step) on update restrict on delete restrict;

Alter table transition add  foreign key (id_step_target) references step (id_step) on update restrict on delete restrict;

Alter table process add  foreign key (id_ts) references type_service (id_ts) on update restrict on delete restrict;

Alter table transition_instance add  foreign key (id_state) references transition_state (id_state) on update restrict on delete restrict;

Alter table log_journal add  foreign key (id_state) references transition_state (id_state) on update restrict on delete restrict;

Alter table relevant_data_process_instance add  foreign key (id_relevant_data) references relevant_data (id_relevant_data) on update restrict on delete restrict;

Alter table attribute_catalog add  foreign key (id_catalog) references catalog (id_catalog) on update restrict on delete restrict;

Alter table relevant_data add  foreign key (id_attribute) references attribute (id_attribute) on update restrict on delete restrict;

Alter table attribute_pool add  foreign key (id_attribute) references attribute (id_attribute) on update restrict on delete restrict;

Alter table attribute_catalog add  foreign key (id_attribute) references attribute (id_attribute) on update restrict on delete restrict;

Alter table pool_instance add  foreign key (id_pool_type) references pool_type (id_pool_type) on update restrict on delete restrict;

Alter table pool_type_process add  foreign key (id_pool_type) references pool_type (id_pool_type) on update restrict on delete restrict;

Alter table attribute_pool add  foreign key (id_pool_type) references pool_type (id_pool_type) on update restrict on delete restrict;

Alter table process_instance add  foreign key (id_pool) references pool_instance (id_pool) on update restrict on delete restrict;

Alter table process_instance add  foreign key (id_process_execution_state) references process_execution_state (id_process_execution_state) on update restrict on delete restrict;

Alter table process add  foreign key (id_monitoring_type) references monitoring_type (id_monitoring_type) on update restrict on delete restrict;

Alter table property_service add Foreign Key (id_property) references property (id_property) on delete  restrict on update  restrict;
Alter table property_component add Foreign Key (id_property) references property (id_property) on delete  restrict on update  restrict;
Alter table component add Foreign Key (id_ts) references type_service (id_ts) on delete  restrict on update  restrict;
Alter table service add Foreign Key (id_component) references component (id_component) on delete  restrict on update  restrict;
Alter table property_component add Foreign Key (id_component) references component (id_component) on delete  restrict on update  restrict;
Alter table service_instance add Foreign Key (id_service) references service (id_service) on delete  restrict on update  restrict;
Alter table property_service add Foreign Key (id_service) references service (id_service) on delete  restrict on update  restrict;
Alter table service_instance add Foreign Key (id_target_service_instance) references service_instance (id_service_instance) on delete  restrict on update  restrict;
Alter table property_service_instance add Foreign Key (id_service_instance) references service_instance (id_service_instance) on delete  restrict on update  restrict;

Alter table variable_instance add Foreign Key (id_process_instance) references process_instance (id_process_instance) on delete  restrict on update  restrict;
Alter table variable_definition add Foreign Key (id_process) references process (id_process) on delete  restrict on update  restrict;
Alter table variable_instance add Foreign Key (id_variable_definition) references variable_definition (id_variable_definition) on delete  restrict on update  restrict;

-- Differences with Spagic 3.2.0

-- New Table to handle Mutex
  
 CREATE TABLE lock_app (
	proc_name Varchar(255) NOT NULL,
	operation_type Varchar(255) NOT NULL,
	id_proc Varchar(255) NOT NULL,
	dt_last_mod Timestamp NOT NULL,
 Primary Key (proc_name,operation_type)) 
	WITHOUT Oids;

-- New Table to handle Node


 CREATE TABLE spagic_node (
	id_node Varchar(255) NOT NULL,
	ip_address Varchar(255),
	host Varchar(255),
	admin_port Int,
	ssl_port Int,
 Primary Key (id_node)) 
 WITHOUT Oids;

-- Association Between Service And Node


 CREATE TABLE service_node (
	id_service Varchar(300) NOT NULL,
	id_node Varchar(255) NOT NULL,
 Primary Key (id_service,id_node)) 
 WITHOUT Oids;

CREATE TABLE form_data (
	id_process_instance Bigint NOT NULL,
	id_step Bigint NOT NULL,
	fdf bytea,
	pdfName Varchar(50),	
 Primary Key (id_process_instance, id_step))
 WITHOUT Oids;

 CREATE TABLE  spagic_token (
	token Varchar(255) NOT NULL,
	dt_expiration Timestamp NOT NULL,
 primary key (token))
 WITHOUT Oids;

Alter table service_node add Foreign Key (id_service) references service (id_service) on delete  restrict on update  restrict;
Alter table service_node add Foreign Key (id_node) references spagic_node (id_node) on delete  restrict on update  restrict;


create unique index uk_host_admin_port on spagic_node (host, admin_port);
create unique index uk_host_ssl_port on spagic_node (host, ssl_port);


Alter table form_data add Foreign Key (id_process_instance) references process_instance (id_process_instance) on delete  restrict on update  restrict;
Alter table form_data add Foreign Key (id_step) references step (id_step) on delete  restrict on update  restrict;

Create table process_configuration (
	id_process_conf Integer NOT NULL,
	id_process Integer NOT NULL ,
	logical_id Varchar(200) NOT NULL,
	is_default Integer NOT NULL,
	conf_data Text NOT NULL,
 primary key (id_process_conf));
 
Alter table process_configuration add Foreign Key (id_process) references process (id_process) on delete  restrict on update  restrict;

